# -*- coding: utf-8 -*-
#
# Helper file to identify manufacturer from OUI
#   http://standards.ieee.org/develop/regauth/oui/oui.txt
#   http://en.wikipedia.org/wiki/Organizationally_unique_identifier
#
# @author: Sreejith Kesavan <sreejithemk@gmail.com>


import re
from pkg_resources import resource_filename


def get_oui_path():
    """
    Return the OUI file path.
    """
    return resource_filename(__name__, 'data/oui.txt')


class OUI(object):
    """ The OUI database
    """

    __shared_state = {}

    oui_file = get_oui_path()

    hex_manuf_pattern = re.compile('\s+(?P<hex>\S+)\s+(\(hex\)|\(base 16\))\s+(?P<manuf>.*)')

    def __init__(self):
        # This is borg. Shares the same state across all instances
        self.__dict__ = self.__shared_state
        self.__parse_oui_file()

    def __parse_oui_file(self):
        # parse the file only once
        if not hasattr(self, '__oui_db'):
            self.__oui_db = {}
            with open(self.oui_file) as f:
                lines = f.readlines()
                for line in lines:
                    match = self.hex_manuf_pattern.match(line)
                    if match:
                        self.__oui_db[match.group('hex')] = match.group('manuf')

    def find_manuf(self, oui):
        return self.__oui_db.get(oui, None)


if __name__ == '__main__':
    oui = OUI()
    print oui.find_manuf('0080E5')
    print oui.find_manuf('00B04A')
    print oui.find_manuf('281878')


# -*- coding: utf-8 -*-
#
# Discover the target host types in the subnet
#
# @author: Sreejith Kesavan <sreejithemk@gmail.com>


import arp
import oui
import ipcalc
import sys


class Discovery(object):
    """ Find out the host types in the Ip range (CIDR)

    NOTE: This finds mac addresses only within the subnet.
    It doesn't fetch mac addresses for routed network ip's.
    """

    def __init__(self):
        self.__arp = arp.ARP()
        self.__oui = oui.OUI()

    def discover(self, address):
        """
        Traverse the IP subnets and return manufacturer info.
        """
        network = ipcalc.Network(address)
        for ip in network:
            ip = str(ip)
            # Ignore broadcast IP Addresses
            if '/' in address and ip == str(network.broadcast()):
                print 'Ignoring broadcast ip: {broadcast}'.format(broadcast=str(network.broadcast()))
                continue

            mac = self.__arp.find_mac(ip)
            if mac:
                if len(mac.split(':')[0]) == 1:
                    mac = '0' + mac
                manuf_str = mac.replace(':', '')[:6].upper()
                manuf = self.__oui.find_manuf(manuf_str)
                if manuf:
                    yield (ip, manuf)


def run():
    if len(sys.argv) < 2:
        print
        print 'Usage:\t\tidiscover <ip-address/cidr>'
        print 'Examples:'
        print '\t\tidiscover 10.73.19.0'
        print '\t\tidiscover 10.74.215/24'
        print
    else:
        addrs = sys.argv[1:]
        d = Discovery()

        try:
            for addr in addrs:
                for ip, manuf in d.discover(addr):
                    print 'IP Address: {ip} Manufacturer: {manuf}'.format(ip=ip, manuf=manuf)
        except KeyboardInterrupt:
            print 'Exiting...'


if __name__ == '__main__':
    run()


# -*- coding: utf-8 -*-

import os

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


def readme(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='idiscover',
    version='1.0.3',
    description='Discover the devices (IP address & Manufacturer information) in a (sub)network',
    long_description=readme('README.md'),
    license='MIT License',
    keywords='Discover IP Address Manufacturer OUI ARP MAC',
    author='Sreejith Kesavan',
    author_email='sreejithemk@gmail.com',
    url='https://github.com/semk/iDiscover',
    download_url='https://github.com/semk/iDiscover/tarball/master',
    install_requires=['ipcalc >= 1.1.2'],
    packages=find_packages(exclude=['ez_setup']),
    package_data={
        'idiscover': ['data/*.txt'],
    },
    include_package_data=True,
    entry_points={
        'console_scripts': ['idiscover = idiscover.discover:run']
    },
    platforms=['any'],
    classifiers=[
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Environment :: Console',
        'Topic :: System :: Networking',
        'Topic :: Utilities'
    ],
    zip_safe=True
)



# -*- coding: utf-8 -*-
#
# Find out the MAC Address of the target host using arp
#
# @author: Sreejith Kesavan <sreejithemk@gmail.com>


import re
import sys
from subprocess import Popen, PIPE


class ARP(object):
    """ Finds the MAC Addresses using ARP

    NOTE: This finds mac addresses only within the subnet.
    It doesn't fetch mac addresses for routed network ips.
    """

    MAC_RE = re.compile(r'(([a-f\d]{1,2}\:){5}[a-f\d]{1,2})')

    def __init__(self):
        if sys.platform == 'win32':
            self.__arp_command_prefix = ['arp', '-a']
        else:
            self.__arp_command_prefix = ['arp', '-n']

    def find_mac(self, ip_address, ping=True):
        if ping:
            # do a ping for an ARP table update
            self.ping(ip_address)

        pid = None
        try:
            pid = Popen(self.__arp_command_prefix + [ip_address], stdout=PIPE)
            out = pid.communicate()[0]
            mac_found = self.MAC_RE.search(out)
            if mac_found:
                mac = mac_found.group(0)
                return mac
        finally:
            if pid:
                pid.stdout.close()

    @staticmethod
    def ping(ip_address):
        """ Ping ip_address
        """
        if sys.platform == 'win32':
            tries_option = '-n'
        else:
            tries_option = '-c'

        # do a single ping
        pid = Popen(['ping', tries_option, '1', ip_address], stdout=PIPE)
        pid.communicate()
        pid.stdout.close()
        if pid.returncode != 0:
            print '{ip} is unreachable'.format(ip=ip_address)


if __name__ == '__main__':
    arp = ARP()
    print arp.find_mac('10.61.187.127')
    print arp.find_mac('10.61.187.145')
    print arp.find_mac('10.73.19.112')

